var platformModule = require("platform");
var frameModule = require("ui/frame");
var view = require("ui/core/view");
var AbsoluteLayout = require("ui/layouts/absolute-layout").AbsoluteLayout;
var StackLayout = require("ui/layouts/stack-layout").StackLayout;
var ScrollView = require("ui/scroll-view").ScrollView;
var Repeater = require("ui/repeater").Repeater;
var Label = require("ui/label").Label;
var helpers = require("./helpers");

var NASlideShow = (function(_super) {
  __extends(NASlideShow, _super);
  function NASlideShow() {
    var _this = this;
    _super.apply(this, arguments);
    
    var _slideIndex = 0, getMeasuredSize, itemsPush, slidePosition = [];
    _this._loaded = false;
    
    // FUNCTION - INIT SLIDESHOW
    _this.init = function(template, props = {}) {
      if(!_this._loaded) {
        var defaults = {
          items: false,
          itemTemplate: "<GridLayout />",
          enableBounce: false,
          fallbackWidth: platformModule.screen.mainScreen.widthDIPs,
          fallbackHeight: platformModule.screen.mainScreen.heightDIPs,
          indicators: {
            position: "bottom",
            size: 10,
            color: "#808080",
            borderWidth: 0,
            borderColor: "#404040",
//            blur: false, // Experimental (not working currently)
            forceVisibility: false,
            disable: false
          }
        };
        for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
        for(var key in defaults.indicators) if(!props.indicators.hasOwnProperty(key)) props.indicators[key] = defaults.indicators[key];

        // THIS
        _this._loaded = true;
        _this.view = template;
        _this.viewsList = {};
        _this.props = props;
        getMeasuredSize = function(xy) {
          if(xy === "width") return template.getMeasuredWidth() || _this.props.fallbackWidth;
            else if(xy === "height") return template.getMeasuredHeight() || _this.props.fallbackHeight;
        };
        
        // 1) Create ScrollView etc
        var scrollView = _this.viewsList.scrollView = new ScrollView();
        scrollView.width = getMeasuredSize("width");
        scrollView.height = getMeasuredSize("height");

        // 2) Create Repeater / StackLayout (container)
        if(props.items) { // Repeater
          var repeater = _this.viewsList.repeater = new Repeater();
          repeater.itemTemplate = props.itemTemplate;

          var items = _this.items = repeater.items = props.items;
          var container = _this.viewsList.container = repeater.itemsLayout = new StackLayout();
//          _this.getSlidesCount = function() { return items.length };
        } else { // StackLayout
          var slides = [];
          template._eachChildView(function(view) { slides.push(view); });

          var container = _this.viewsList.container = scrollView.content = new StackLayout();
//          _this.getSlidesCount = function() { return container.getChildrenCount() };

          template.removeChildren();
        }

        // 3) Set classes and other properties
        template.className = "NASlideShow";
        scrollView.orientation = "horizontal";
        if(props.items) repeater.className = "NASlideShow-repeater";
        container.className = "NASlideShow-container";
        container.orientation = "horizontal";

        // 4) Insert views
        if(props.items) { // Repeater
          scrollView.content = repeater;
        } else { // No Repeater
          slides.forEach(function(view, index) {
            container.insertChild(view, index);
          });
          scrollView.content = container;
        }
        template.insertChild(scrollView, 0);

        // 5) Create indicators view
        if(!props.indicators.forceVisibility && _this.getSlidesCount() < 2) props.indicators.disable = true;

        var indicators = _this.viewsList.indicators = new StackLayout();
        indicators.className = "NASlideShow-indicators";
        indicators.height = props.indicators.size + 10;
        indicators.orientation = "horizontal";
        indicators.isUserInteractionEnabled = false;
        indicators.translateY = (props.indicators.position === "top" ? 15 : -15);
        if(props.indicators.disable) indicators.visibility = "collapsed";

        setTimeout(function() {
          AbsoluteLayout.setTop(indicators, (props.indicators.position === "top" ? 0 : getMeasuredSize("height") - indicators.height));
          AbsoluteLayout.setLeft(indicators, (getMeasuredSize("width") - indicators.getMeasuredWidth()) / 2);
        }, 0);

        template.insertChild(indicators, 1);

        var appendIndicator = function(i) {
          var view = container.getChildAt(i);
          if(view) view.width = getMeasuredSize("width");
          slidePosition.push(getMeasuredSize("width") * i);

          if(!props.indicators.disable) {
            var dot = new StackLayout();
            dot.className = "NASlideShow-indicators-dot";
            dot.width = dot.height = props.indicators.size;
            dot.margin = 5;
//            dot.backgroundColor = helpers.color(props.indicators.color, (i === 0 ? 0.75 : 0.25));
            dot.backgroundColor = props.indicators.color;
            dot.borderRadius = props.indicators.size / 2;
            dot.clipToBounds = false;
            dot.opacity = (i === 0 ? 0.75 : 0.25);
            
            if(props.indicators.borderWidth) { // Border
              dot.borderWidth = props.indicators.borderWidth;
              dot.borderColor = props.indicators.borderColor;
            }
            
//            if(props.indicators.blur) { // Blur
//              if(frameModule.topmost().ios) { // iOS
//                var blurView = UIVisualEffectView.alloc().initWithEffect(UIBlurEffect.effectWithStyle(UIBlurEffectStyleLight));
//                blurView.frame = { origin: { x: 0, y: 0 }, size: { width: dot.ios.frame.size.width, height: dot.ios.frame.size.height } };
//                blurView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
//                dot.ios.addSubview(blurView);
//                dot.ios.sendSubviewToBack(blurView);
//              }
//            }

            indicators.insertChild(dot, i);
          }
        };

        if(props.indicators.forceVisibility) appendIndicator(0);
        for(var i = 0; i < _this.getSlidesCount(); i++) {
          if((props.indicators.forceVisibility && i > 0) || !props.indicators.forceVisibility) appendIndicator(i);
        }

        if(!props.indicators.disable) {
          var scrollX = 0, direction = null;
          var currentSlideIndex = 0, previousSlideIndex = null;
          var incomingDot = null;

          scrollView.on("scroll", function(args) {
            direction = (scrollX > args.scrollX ? "left" : (scrollX === args.scrollX ? null : "right"));

            if(direction === "left") incomingDot = indicators.getChildAt(Math.max(currentSlideIndex, 1) - 1);
              else if(direction === "right") incomingDot = indicators.getChildAt(Math.min(currentSlideIndex, 1) + 1);
              else incomingDot = indicators.getChildAt(currentSlideIndex);

            // When scroll has finished
            if(slidePosition.indexOf(args.scrollX) >= 0) {
              slidePosition.forEach(function(value, index, array) {
                if(args.scrollX === value) {
                  previousSlideIndex = currentSlideIndex;
                  currentSlideIndex = index;
                }

                if(index === array.length-1) { // Last
                  if(previousSlideIndex !== currentSlideIndex) {
                    var previousIndicator = indicators.getChildAt(previousSlideIndex);
                    var currentIndicator = indicators.getChildAt(currentSlideIndex);

                    previousIndicator.animate({ opacity: 0.25, duration: 100 });
                    currentIndicator.animate({ opacity: 0.75, duration: 100 });
                    
                    _slideIndex = currentSlideIndex;

                    // CREATE EVENT LISTENER
                    var eventData = {
                      eventName: "slideChange",
                      object: _this,
                      index: currentSlideIndex,
                      length: _this.getSlidesCount()
                    };
                    _this.notify(eventData);
                  }
                }
              });
            }

            scrollX = args.scrollX;
          });
        }

        if(frameModule.topmost().ios) { // iOS
          var nativeScrollView = scrollView.ios;
          
          nativeScrollView.pagingEnabled = true;
          if(!props.enableBounce) nativeScrollView.bounces = false;
          nativeScrollView.showsHorizontalScrollIndicator = false;
          nativeScrollView.showsVerticalScrollIndicator = false;
        } else if(frameModule.topmost().android) { // Android
          
        }
        
        // OBSERVE ITEMS PUSH
        if(_this.items) {
          _this.items.on("propertyChange", function(args) {
            if(!itemsPush) appendSlide();
          });
        }
      }
    };
    
    
    // FUNCTION - APPEND SLIDE
    _this.appendSlide = function(slideView, props = {}) {
      if(_this.items) { // Repeater
        if(!itemsPush) {
          itemsPush = true;
          _this.items.push(slideView || null);
          if(_this.getSlidesCount() === 1 && !slideView) _this.items.push(null);
          
          appendSlide(slideView, props);
        }
      } else { // No Repeater
        appendSlide(slideView, props);
      }
    };
    
    function appendSlide(slideView, props = {}) {
      var defaults = {
        indicator: {
          icon: false,
          fontFamily: false,
          float: false
        }
      };
      for(var key in defaults) if(!props.hasOwnProperty(key)) props[key] = defaults[key];
      for(var key in defaults.indicator) if(!props.indicator.hasOwnProperty(key)) props.indicator[key] = defaults.indicator[key];
      
      itemsPush = undefined;
      if(_this.items && slideView && (slideView.ios || slideView.android)) return false;
      
      // INDICATOR
      if(_this.getSlidesCount() > 0) {
        var indicator = new StackLayout();
        indicator.className = (props.indicator.icon ? "NASlideShow-indicators-icon" : "NASlideShow-indicators-dot");
        indicator.width = indicator.height = _this.props.indicators.size;
        indicator.margin = 5;
        if(!props.indicator.icon) indicator.backgroundColor = _this.props.indicators.color;
        indicator.borderRadius = _this.props.indicators.size / 2;
        indicator.opacity = 0.25;
        indicator.clipToBounds = false;
            
        if(_this.props.indicators.borderWidth && !props.indicator.icon) { // Border
          indicator.borderWidth = _this.props.indicators.borderWidth;
          indicator.borderColor = _this.props.indicators.borderColor;
        }
        
//        if(_this.props.indicators.blur) { // Blur
//          if(frameModule.topmost().ios) { // iOS
//            var blurView = UIVisualEffectView.alloc().initWithEffect(UIBlurEffect.effectWithStyle(UIBlurEffectStyleLight));
//            blurView.frame = { origin: { x: 0, y: 0 }, size: { width: indicator.ios.frame.size.width, height: indicator.ios.frame.size.height } };
//            blurView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
//            indicator.ios.addSubview(blurView);
//            indicator.ios.sendSubviewToBack(blurView);
//          }
//        }

        if(props.indicator.icon) {
//          indicator.opacity = 0.25;
          if(props.indicator.float) indicator.marginRight = -(_this.props.indicators.size + 5);

          var label = new Label();
          label.text = props.indicator.icon;
          label.color = _this.props.indicators.color;
          label.marginTop = -3;
          label.marginLeft = -3;
          label.fontSize = _this.props.indicators.size * 2;
          if(props.fontFamily) label.fontFamily = props.fontFamily;

          indicator.insertChild(label);
        }

        var indicators = _this.viewsList.indicators;
        indicators.insertChild(indicator, indicators.getChildrenCount());
      }

      // SLIDE VIEW
      var formula = (_this.getSlidesCount() - (_this.items ? 1 : 0));
      slidePosition.push(getMeasuredSize("width") * formula);
      
      if(_this.items) { // Repeater
        _this.viewsList.container._eachChildView(function(view) {
          view.width = getMeasuredSize("width");
        });
      } else { // No Repeater
        slideView.width = getMeasuredSize("width");
        _this.viewsList.container.insertChild(slideView, _this.getSlidesCount());
      }

      return _this;
    }
    
    
    // FUNCTION - GET SLIDES COUNT
    _this.getSlidesCount = function() {
      if(_this.props.items) return _this.items.length;
        else return _this.viewsList.container.getChildrenCount();
    };
    
    
    // FUNCTION - GET SLIDE INDEX
    _this.getSlideIndex = function() {
      return _slideIndex;
    };
    
  }
  
  return NASlideShow;
})(view.View);

module.exports = NASlideShow;